@@ -3,6 +3,8 @@ import logging
 
 import numpy as np
 
+from flex.extensions.bindata import MultipleDataExtension
+
 from ..data_structure import (
     CollectionFactory,
     Collection,
@@ -41,12 +43,12 @@ class Atmosphere(Collection):
         ("vturb", 0, absolute, this, "float: turbulence velocity in km/s"),
         ("lonh", 0, asfloat, this, "float: ?"),
         ("source", None, asstr, this, "str: datafile name of this data"),
-        ("method", None, lowercase(oneof(None, "grid", "embedded")), this, 
+        ("method", "grid", lowercase(oneof("grid", "embedded")), this, 
             "str: whether the data source is a grid or a fixed atmosphere"),
-        ("geom", None, uppercase(oneof(None, "PP", "SPH")), this,
+        ("geom", "PP", uppercase(oneof("PP", "SPH", None)), this,
             "str: the geometry of the atmopshere model"),
         ("radius", 0, asfloat, this, "float: radius of the spherical model"),
-        ("height", 0, asfloat, this, "float: height of the spherical model"),
+        ("height", None, array(None, "f8"), this, "array: height of the spherical model"),
         ("opflag", [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0], array(20, int), this,
             "array of size (20,): opacity flags"),
         ("wlstd", 5000, asfloat, this, "float: wavelength standard deviation"),
@@ -70,6 +72,8 @@ class Atmosphere(Collection):
     ]
     # fmt: on
 
+    # TODO: pick the right geometry for the grid, based on whether it has height/radius values or not
+
     def __init__(self, **kwargs):
         monh = kwargs.pop("monh", kwargs.pop("feh", 0))
         abund = kwargs.pop("abund", "empty")
@@ -88,7 +92,7 @@ class Atmosphere(Collection):
 
     @property
     def names(self):
-        return self._names
+        return self._names + ["monh"]
 
     @property
     def dtype(self):
@@ -107,6 +111,36 @@ class Atmosphere(Collection):
     def ndep(self, value):
         pass
 
+    def _save(self):
+        data = {}
+        ext2 = self.abund._save()
+        header = ext2.header
+        data["abund"] = ext2.data
+        header["abund_format"] = header["type"]
+        del header["type"]
+
+        for name in self._names:
+            value = self[name]
+
+            if isinstance(value, np.ndarray):
+                data[name] = value
+            elif value is None or isinstance(value, Abund):
+                pass
+            else:
+                # if isinstance(value, (np.floating, np.integer, np.str, float, int, str)):
+                header[name] = value
+
+        ext = MultipleDataExtension(header, data)
+
+        return ext
+
+    @classmethod
+    def _load(cls, ext):
+        header = ext.header
+        header.update(ext.data)
+        obj = cls(**header)
+        return obj
+
 
 @CollectionFactory
 class AtmosphereGrid(np.recarray):
@@ -119,7 +153,7 @@ class AtmosphereGrid(np.recarray):
     # fmt: off
     _fields = [
         ("source", None, asstr, this, "str: datafile name of this data"),
-        ("method", None, lowercase(oneof(None, "grid", "embedded")), this, 
+        ("method", "grid", lowercase(oneof("grid", "embedded")), this, 
             "str: whether the data source is a grid or a fixed atmosphere"),
         ("geom", None, uppercase(oneof(None, "PP", "SPH")), this,
             "str: the geometry of the atmopshere model"),
@@ -141,7 +175,7 @@ class AtmosphereGrid(np.recarray):
             ("vturb", "f4"),
             ("lonh", "f4"),
             ("radius", "f4"),
-            ("height", "f4"),
+            ("height", f"({npoints},)f4"),
             ("wlstd", "f4"),
             ("opflag", "(20,)i4"),
             ("abund", "(99,)f4"),
@@ -174,10 +208,11 @@ class AtmosphereGrid(np.recarray):
         self.interp = getattr(self, "interp", "TAU")
         self.depth = getattr(self, "depth", "RHOX")
         self.source = getattr(self, "source", "")
-        self.geom = getattr(self, "geom", "pp")
+        self.geom = getattr(self, "geom", "PP")
         self.citation_info = getattr(self, "citation_info", "")
         self.method = getattr(self, "method", "grid")
         self.abund_format = getattr(self, "abund_format", "sme")
+        self.wlstd = getattr(self, "wlstd", 5000)
 
     def __getitem__(self, key):
         """ Overwrite the getitem routine, so we keep additional
